@@ -1,6 +1,7 @@
 //! Encoders
 use crate::{
-    error::Error, Color, ColorLinear, FaceAttrs, TerminalCaps, TerminalColor, TerminalCommand,
+    decoder::KEYBOARD_LEVEL, error::Error, Color, ColorLinear, DecMode, FaceAttrs, TerminalCaps,
+    TerminalColor, TerminalCommand,
 };
 use std::{cmp::Ordering, io::Write};
 
@@ -29,6 +30,13 @@ impl TTYEncoder {
     pub fn new(caps: TerminalCaps) -> Self {
         Self { caps }
     }
+
+    fn kitty_level<W: Write>(&self, mut out: W, level: usize) -> Result<(), Error> {
+        if self.caps.kitty_keyboard {
+            write!(out, "\x1b[={}u", level)?;
+        }
+        Ok(())
+    }
 }
 
 impl Encoder for TTYEncoder {
@@ -40,8 +48,17 @@ impl Encoder for TTYEncoder {
 
         match cmd {
             DecModeSet { enable, mode } => {
+                // kitty keyboard level maintained separetely for alt-screen
+                if !enable && mode == DecMode::AltScreen {
+                    self.kitty_level(&mut out, 0)?;
+                }
+
                 let flag = if enable { "h" } else { "l" };
                 write!(out, "\x1b[?{}{}", mode as usize, flag)?;
+
+                if enable && mode == DecMode::AltScreen {
+                    self.kitty_level(out, KEYBOARD_LEVEL)?;
+                }
             }
             DecModeGet(mode) => {
                 write!(out, "\x1b[?{}$p", mode as usize)?;
@@ -130,6 +147,9 @@ impl Encoder for TTYEncoder {
             DeviceAttrs => {
                 write!(out, "\x1b[c")?;
             }
+            KeyboardLevel(level) => {
+                self.kitty_level(out, level)?;
+            }
         }
 
         Ok(())
@@ -253,7 +273,7 @@ pub fn color_sgr_encode<C: Color, W: Write>(
             } else {
                 out.write_all(b";48")?;
             }
-            write!(out, ";2;{};{};{}", r, g, b)?;
+            write!(out, ":2:{}:{}:{}", r, g, b)?;
         }
         ColorDepth::EightBit => {
             let color: ColorLinear = color.into();
@@ -285,10 +305,11 @@ pub fn color_sgr_encode<C: Color, W: Write>(
         }
         ColorDepth::Gray => {
             let luma = color.luma();
-            let index = match nearest(luma, &[0.0, 0.7, 1.0]) {
+            let index = match nearest(luma, &[0.0, 0.33, 0.66, 1.0]) {
                 0 => 30,
                 1 => 90,
-                _ => 37,
+                2 => 37,
+                _ => 97,
             };
             let index = if foreground { index } else { index + 10 };
             write!(out, ";{}", index)?;
@@ -320,10 +341,10 @@ mod tests {
     }
 
     #[test]
-    fn test_sgr() -> Result<(), Error> {
+    fn test_gray_sgr() -> Result<(), Error> {
         let mut encoder = TTYEncoder::new(TerminalCaps {
             depth: ColorDepth::Gray,
-            glyphs: false,
+            ..TerminalCaps::default()
         });
         let mut out = Vec::new();
         encoder.encode(
@@ -333,7 +354,7 @@ mod tests {
 
         assert_eq!(
             std::str::from_utf8(out.as_ref()).as_deref(),
-            Ok("\x1b[00;30;47m")
+            Ok("\x1b[00;30;107m")
         );
         Ok(())
     }
