@@ -7,8 +7,8 @@
 use crate::{
     common::{clamp, Rnd},
     encoder::Base64Encoder,
-    Blend, Color, Error, Position, Shape, Size, Surface, SurfaceMut, SurfaceOwned, Terminal,
-    TerminalCommand, TerminalEvent, TerminalSize, RGBA,
+    Blend, Color, Error, Position, Shape, Size, Surface, SurfaceMut, SurfaceOwned, TerminalEvent,
+    TerminalSize, RGBA,
 };
 use flate2::{write::ZlibEncoder, Compression};
 use std::{
@@ -19,9 +19,8 @@ use std::{
     iter::FromIterator,
     ops::{Add, AddAssign, Mul},
     sync::Arc,
-    time::Duration,
 };
-use tracing::{debug_span, info};
+use tracing::debug_span;
 
 const IMAGE_CACHE_SIZE: usize = 134217728; // 128MB
 
@@ -177,8 +176,8 @@ impl Surface for Image {
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum ImageHandlerKind {
     Kitty,
-    ITerm,
     Sixel,
+    ITerm,
     Dummy,
 }
 
@@ -234,59 +233,6 @@ impl<'a> ImageHandler for Box<dyn ImageHandler> {
     }
 }
 
-/// Detect appropriate image handler for provided termainl
-pub fn image_handler_detect(
-    term: &mut dyn Terminal,
-) -> Result<Box<dyn ImageHandler + 'static>, Error> {
-    // NOTE: will not work if kitty handler has already been set, as it would
-    // consume event generated by terminal inside `ImageHandler::handle`
-    while term.poll(Some(Duration::new(0, 0)))?.is_some() {}
-    // Send kitty query and DA1 request
-    term.write_all(b"\x1b_Ga=q,i=31,s=1,v=1,f=24;AAAA\x1b\\")?; // kitty image
-    term.write_all(b"\x1b]11;?\x1b\\")?; // background color
-    term.execute(TerminalCommand::DeviceAttrs)?; // we expect to see response at least for this one or there will be 1s deley
-    let mut handler: Option<Box<dyn ImageHandler + 'static>> = None;
-    let mut bg: Option<RGBA> = None;
-    for _ in 0..3 {
-        match term.poll(Some(Duration::from_secs(1)))? {
-            Some(TerminalEvent::KittyImage { .. }) => {
-                handler.replace(Box::new(KittyImageHandler::new()));
-            }
-            Some(TerminalEvent::DeviceAttrs(attrs)) => {
-                // 4 - attribute indicates sixel support
-                if handler.is_none() && attrs.contains(&4) {
-                    handler.replace(Box::new(SixelImageHandler::new(bg)));
-                }
-                break;
-            }
-            Some(TerminalEvent::Color { color, .. }) => {
-                bg.replace(color);
-            }
-            _ => continue,
-        }
-    }
-    // drain terminal
-    term.drain().count();
-    // check environment for handler override
-    if let Ok(name) = std::env::var("SURF_N_TERM_IMG") {
-        match name.as_ref() {
-            "iterm" => {
-                handler.replace(Box::new(ItermImageHandler::new()));
-            }
-            "kitty" => {
-                handler.replace(Box::new(KittyImageHandler::new()));
-            }
-            "sixel" => {
-                handler.replace(Box::new(SixelImageHandler::new(bg)));
-            }
-            _ => {}
-        }
-    }
-    let handler = handler.unwrap_or_else(|| Box::new(DummyImageHandler));
-    info!("image handler detected: {:?}", handler.kind());
-    Ok(handler)
-}
-
 /// Image handler which ignores requests
 pub struct DummyImageHandler;
 
@@ -322,7 +268,7 @@ pub struct ItermImageHandler {
 }
 
 impl ItermImageHandler {
-    fn new() -> Self {
+    pub fn new() -> Self {
         Self {
             imgs: lru::LruCache::unbounded(),
             size: 0,
@@ -330,6 +276,12 @@ impl ItermImageHandler {
     }
 }
 
+impl Default for ItermImageHandler {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl ImageHandler for ItermImageHandler {
     fn kind(&self) -> ImageHandlerKind {
         ImageHandlerKind::ITerm
@@ -438,8 +390,11 @@ impl ImageHandler for KittyImageHandler {
             }
             let payload = payload_write.finish()?.finish()?;
 
-            // data needs to be transfered in chunks
-            let chunks = payload.chunks(4095);
+            // NOTE:
+            //  - data needs to be transfered in chunks
+            //  - chunks should be multiple of 4, otherwise kitty complains that it is not
+            //    valid base64 encoded data.
+            let chunks = payload.chunks(4096);
             let count = chunks.len();
             for (index, chunk) in chunks.enumerate() {
                 // control data
