@@ -182,6 +182,7 @@ impl<'a, T: Terminal + ?Sized> Terminal for &'a mut T {
 pub struct TerminalCaps {
     pub depth: ColorDepth,
     pub glyphs: bool,
+    pub kitty_keyboard: bool,
 }
 
 impl Default for TerminalCaps {
@@ -189,6 +190,7 @@ impl Default for TerminalCaps {
         Self {
             depth: ColorDepth::EightBit,
             glyphs: false,
+            kitty_keyboard: false,
         }
     }
 }
@@ -285,6 +287,8 @@ pub enum TerminalCommand {
     Title(String),
     /// [Primary Device Attributes](https://vt100.net/docs/vt510-rm/DA1.html)
     DeviceAttrs,
+    /// Set kitty keyboard protocol level
+    KeyboardLevel(usize),
 }
 
 /// Kind of terminal color
@@ -310,8 +314,6 @@ pub enum DecMode {
     MouseSGR = 1006,
     /// Alternative screen mode
     AltScreen = 1049,
-    /// Kitty keyboard mode https://sw.kovidgoyal.net/kitty/protocol-extensions.html
-    KittyKeyboard = 2017,
     /// Synchronized output https://gist.github.com/christianparpart/d8a62cc1ab659194337d73e399004036
     SynchronizedOutput = 2026,
 }
@@ -342,7 +344,7 @@ impl DecMode {
             MouseMotions,
             MouseSGR,
             AltScreen,
-            KittyKeyboard,
+            SynchronizedOutput,
         ]
         .iter()
         {
@@ -410,6 +412,8 @@ pub enum TerminalEvent {
     },
     /// Kitty image result
     KittyImage { id: u64, error: Option<String> },
+    /// Kitty keyboard level
+    KeyboardLevel(usize),
     /// Terminal have been woken by waker object
     Wake,
     /// Termcap/Terminfo repsponse to XTGETTCAP
