@@ -1,5 +1,6 @@
 //! Unix systems specific `Terminal` implementation.
 use crate::common::IOQueue;
+use crate::decoder::KEYBOARD_LEVEL;
 use crate::encoder::ColorDepth;
 use crate::image::ImageHandlerKind;
 use crate::{
@@ -133,6 +134,7 @@ impl UnixTerminal {
         };
 
         capabilities_detect(&mut term)?;
+        term.execute(TerminalCommand::KeyboardLevel(KEYBOARD_LEVEL))?;
         Ok(term)
     }
 
@@ -208,6 +210,7 @@ impl UnixTerminal {
                 enable: true,
                 mode: DecMode::AutoWrap,
             },
+            TerminalCommand::KeyboardLevel(0),
         ];
         epilogue
             .iter()
@@ -241,13 +244,17 @@ const GET_TERM_SIZE: &[u8] = b"\x1b[18t\x1b[14t";
 
 /// Detect and set terminal capabilities
 fn capabilities_detect(term: &mut UnixTerminal) -> Result<(), Error> {
+    if let Ok("linux") | Ok("dumb") = std::env::var("TERM").as_deref() {
+        // do not try to query anything on dumb terminals
+        warn!("[detected] dump terminal");
+        term.capabilities.depth = ColorDepth::Gray;
+        term.encoder = TTYEncoder::new(term.capabilities.clone());
+        return Ok(());
+    }
     let mut caps = TerminalCaps::default();
     if let Ok("truecolor") | Ok("24bit") = std::env::var("COLORTERM").as_deref() {
         caps.depth = ColorDepth::TrueColor;
     }
-    if let Ok("linux") | Ok("dumb") = std::env::var("TERM").as_deref() {
-        caps.depth = ColorDepth::EightBit;
-    }
 
     // drain all pending events
     term.drain().count();
@@ -271,6 +278,9 @@ fn capabilities_detect(term: &mut UnixTerminal) -> Result<(), Error> {
     // Some terminals return incomplete size info with ioctl
     term.write_all(GET_TERM_SIZE)?;
 
+    // Detect kitty keyboard protocol support
+    write!(term, "\x1b[?u")?;
+
     // DA1 - sync and sixel info
     // Device Attribute comand is used as "sync" event, it is supported
     // by most terminals, at least in its basic form, so we expect to
@@ -307,6 +317,10 @@ fn capabilities_detect(term: &mut UnixTerminal) -> Result<(), Error> {
             Some(TerminalEvent::Size(size)) => {
                 size_escape = size;
             }
+            Some(TerminalEvent::KeyboardLevel(_)) => {
+                debug!("[detect] kitty keyboard protocol");
+                caps.kitty_keyboard = true;
+            }
             Some(event) => {
                 warn!("unexpected event during detection: {:?}", event);
                 continue;
